@@ -1,5 +1,5 @@
 /*
- * powernowd.c: (c) 2003-2005 John Clemens <clemej@alum.rpi.edu>
+ * powernowd.c: (c) 2003-2008 John Clemens <clemej@alum.rpi.edu>
  *
  * Released under the GNU General Public License. See the LICENSE file
  * included with this file.
@@ -33,6 +33,9 @@
  *          braindead think-o in the multi-cpu speed change logic.  Random
  *          other small cleanups/suggestions from debian bugtrack.  Fix
  *          Segfault on exit on SMP systems.
+ *   v1.00, Small fixes and cleanups for a 1.00 release.  s/strtol/strtoll to
+ *          handle large uptimes (from Debian/Ubuntu package).  Fix harmless 
+ *          buffer overrun (non-exploitable).  valgrind clean.
  *
  *   
  * * Contributions from:
@@ -41,7 +44,7 @@
  * 	Wolfgang Tremmel <tremmel@garf.de>
  * 	Hans Ulrich Niedermann <kdev@n-dimensional.de>
  * 	Bdale Garbee, Vince Weaver
- * 	-- And many others who have sent me patches over the past 3 years,
+ * 	-- And many others who have sent me patches over the years,
  * 	   some of whom I've never even acknowledged.  Just know that your 
  * 	   support is appreciated, even if I didn't merge in your ideas.  
  * 	   Thank you.
@@ -141,15 +144,15 @@ time_t start_time = 0;
 #define SYSFS_TREE "/sys/devices/system/cpu/"
 #define SYSFS_SETSPEED "scaling_setspeed"
 
-#define VERSION	"0.97"
+#define VERSION	"1.00"
 
 void help(void)
 {
 	
-	printf("PowerNow Daemon v%s, (c) 2003-2005 John Clemens\n", VERSION);
+	printf("PowerNow Daemon v%s, (c) 2003-2008 John Clemens\n", VERSION);
 	printf("Daemon to control the speed and voltage of cpus.\n\n");
 	printf("This is a simple client to the CPUFreq driver, and uses\n");
-	printf("linux kernel v2.5 sysfs interface.  You need a supported\n");
+	printf("linux kernel v2.5+ sysfs interface.  You need a supported\n");
 	printf("cpu, and a kernel that supports sysfs to run this daemon.\n");
 	printf("\nAvailable Options:\n");
 	printf(" 	-h	Print this help message\n");
@@ -230,13 +233,13 @@ int get_stat(cpuinfo_t *cpu)
 
 	memcpy(cpu->last_reading, cpu->reading, sizeof(cpustats_t));
 	
-	cpu->reading->user = strtol(p2, &p2, 10);
-	cpu->reading->mynice = strtol(p2, &p2, 10);
-	cpu->reading->system = strtol(p2, &p2, 10);
-	cpu->reading->idle = strtol(p2, &p2, 10);
-	cpu->reading->iowait = strtol(p2, &p2, 10);
-	cpu->reading->irq = strtol(p2, &p2, 10);
-	cpu->reading->softirq = strtol(p2, &p2, 10);
+	cpu->reading->user = strtoll(p2, &p2, 10);
+	cpu->reading->mynice = strtoll(p2, &p2, 10);
+	cpu->reading->system = strtoll(p2, &p2, 10);
+	cpu->reading->idle = strtoll(p2, &p2, 10);
+	cpu->reading->iowait = strtoll(p2, &p2, 10);
+	cpu->reading->irq = strtoll(p2, &p2, 10);
+	cpu->reading->softirq = strtoll(p2, &p2, 10);
 
 	return 0;
 }
@@ -390,7 +393,7 @@ int faked_compare(const void *a, const void *b)
  */
 int get_per_cpu_info(cpuinfo_t *cpu, int cpuid)
 {
-	char cpustr[100], scratch[100], tmp[10], *p1;
+	char cpustr[100], scratch[100], tmp[11], *p1;
 	int fd, err;
 	unsigned long temp;
 	
@@ -602,6 +605,7 @@ void terminate(int signum)
 		free(cpu->sysfs_dir);
 		free(cpu->last_reading);
 		free(cpu->reading);
+		free(cpu->freq_table);
 		free(cpu);
 	}
 	free(all_cpus);
@@ -676,7 +680,7 @@ int determine_threads_per_core(int ncpus)
 	 * cpus on the system.
 	 */
 	strncpy(filename, SYSFS_TREE, 30);
-	sprintf(filename, "%scpu0/cpufreq/affected_cpus", filename);
+	strncat(filename, "cpu0/cpufreq/affected_cpus", 99-strlen(filename));
 	
 	/* 
 	 * OK, the funkiest system I can think of right now is
@@ -855,7 +859,7 @@ int main(int argc, char **argv)
 		openlog("powernowd", LOG_AUTHPRIV|LOG_PERROR, LOG_DAEMON);
 	
 	/* My ego's pretty big... */
-	pprintf(0,"PowerNow Daemon v%s, (c) 2003-2006 John Clemens\n", 
+	pprintf(0,"PowerNow Daemon v%s, (c) 2003-2008 John Clemens\n", 
 			VERSION);
 
 	/* are we root?? */
